@@ -15,16 +15,15 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "Hardpoint.h"
 
-#include <list>
 #include <map>
 #include <vector>
 
 class Command;
-class Effect;
 class Outfit;
 class Point;
 class Projectile;
 class Ship;
+class Visual;
 
 
 
@@ -40,7 +39,7 @@ class Ship;
 class Armament {
 public:
 	// Add a gun or turret hard-point.
-	void AddGunPort(const Point &point, const Outfit *outfit = nullptr);
+	void AddGunPort(const Point &point, const Angle &angle, bool isParallel, const Outfit *outfit = nullptr);
 	void AddTurret(const Point &point, const Outfit *outfit = nullptr);
 	// This must be called after all the outfit data is loaded. If you add more
 	// of a given weapon than there are slots for it, the extras will not fire.
@@ -65,9 +64,9 @@ public:
 	void Aim(const Command &command);
 	// Fire the given weapon, if it is ready. If it did not fire because it is
 	// not ready, return false.
-	void Fire(int index, Ship &ship, std::list<Projectile> &projectiles, std::list<Effect> &effects);
+	void Fire(int index, Ship &ship, std::vector<Projectile> &projectiles, std::vector<Visual> &visuals);
 	// Fire the given anti-missile system.
-	bool FireAntiMissile(int index, Ship &ship, const Projectile &projectile, std::list<Effect> &effects);
+	bool FireAntiMissile(int index, Ship &ship, const Projectile &projectile, std::vector<Visual> &visuals);
 	
 	// Update the reload counters.
 	void Step(const Ship &ship);
