@@ -20,10 +20,12 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Politics.h"
 #include "ShipEvent.h"
 
+#include <algorithm>
+
 using namespace std;
 
 namespace {
-	static unsigned nextID = 0;
+	unsigned nextID = 0;
 }
 
 
@@ -48,16 +50,26 @@ Government::Government()
 void Government::Load(const DataNode &node)
 {
 	if(node.Size() >= 2)
+	{
 		name = node.Token(1);
+		if(displayName.empty())
+			displayName = name;
+	}
 	
 	for(const DataNode &child : node)
 	{
-		if(child.Token(0) == "swizzle" && child.Size() >= 2)
+		if(child.Token(0) == "display name" && child.Size() >= 2)
+			displayName = child.Token(1);
+		else if(child.Token(0) == "swizzle" && child.Size() >= 2)
 			swizzle = child.Value(1);
 		else if(child.Token(0) == "color" && child.Size() >= 4)
 			color = Color(child.Value(1), child.Value(2), child.Value(3));
 		else if(child.Token(0) == "player reputation" && child.Size() >= 2)
 			initialPlayerReputation = child.Value(1);
+		else if(child.Token(0) == "crew attack" && child.Size() >= 2)
+			crewAttack = max(0., child.Value(1));
+		else if(child.Token(0) == "crew defense" && child.Size() >= 2)
+			crewDefense = max(0., child.Value(1));
 		else if(child.Token(0) == "attitude toward")
 		{
 			for(const DataNode &grand : child)
@@ -97,6 +109,10 @@ void Government::Load(const DataNode &node)
 			bribe = child.Value(1);
 		else if(child.Token(0) == "fine" && child.Size() >= 2)
 			fine = child.Value(1);
+		else if(child.Token(0) == "enforces" && child.HasChildren())
+			enforcementZones.emplace_back(child);
+		else if(child.Token(0) == "enforces" && child.Size() == 2 && child.Token(1) == "all")
+			enforcementZones.clear();
 		else if(child.Token(0) == "death sentence" && child.Size() >= 2)
 			deathSentence = GameData::Conversations().Get(child.Token(1));
 		else if(child.Token(0) == "friendly hail" && child.Size() >= 2)
@@ -124,8 +140,23 @@ void Government::Load(const DataNode &node)
 
 
 
-// Get the name of this government.
+// Get the display name of this government.
 const string &Government::GetName() const
+{
+	return displayName;
+}
+
+
+
+// Set / Get the name used for this government in the data files.
+void Government::SetName(const string &trueName)
+{
+	this->name = trueName;
+}
+
+
+
+const string &Government::GetTrueName() const
 {
 	return name;
 }
@@ -200,6 +231,30 @@ double Government::GetFineFraction() const
 
 
 
+// Returns true if this government has no enforcement restrictions, or if the
+// indicated system matches at least one enforcement zone.
+bool Government::CanEnforce(const System *system) const
+{
+	for(const LocationFilter &filter : enforcementZones)
+		if(filter.Matches(system))
+			return true;
+	return enforcementZones.empty();
+}
+
+
+
+// Returns true if this government has no enforcement restrictions, or if the
+// indicated planet matches at least one enforcement zone.
+bool Government::CanEnforce(const Planet *planet) const
+{
+	for(const LocationFilter &filter : enforcementZones)
+		if(filter.Matches(planet))
+			return true;
+	return enforcementZones.empty();
+}
+
+
+
 const Conversation *Government::DeathSentence() const
 {
 	return deathSentence;
@@ -312,3 +367,17 @@ void Government::SetReputation(double value) const
 {
 	GameData::GetPolitics().SetReputation(this, value);
 }
+
+
+
+double Government::CrewAttack() const
+{
+	return crewAttack;
+}
+
+
+
+double Government::CrewDefense() const
+{
+	return crewDefense;
+}
