@@ -17,14 +17,16 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "ConversationPanel.h"
 #include "DataFile.h"
 #include "Dialog.h"
+#include "text/DisplayText.h"
 #include "Files.h"
 #include "FillShader.h"
-#include "Font.h"
-#include "FontSet.h"
-#include "Format.h"
+#include "text/Font.h"
+#include "text/FontSet.h"
+#include "text/Format.h"
 #include "GameData.h"
 #include "Information.h"
 #include "Interface.h"
+#include "text/layout.hpp"
 #include "MainPanel.h"
 #include "Messages.h"
 #include "PlayerInfo.h"
@@ -32,9 +34,14 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Rectangle.h"
 #include "ShipyardPanel.h"
 #include "StarField.h"
+#include "StartConditionsPanel.h"
+#include "text/truncate.hpp"
 #include "UI.h"
 
+#include "gl_header.h"
+
 #include <algorithm>
+#include <stdexcept>
 
 using namespace std;
 
@@ -53,6 +60,34 @@ namespace {
 #endif
 		return string(buf, strftime(buf, BUF_SIZE, FORMAT, date));
 	}
+	
+	// Extract the date from this pilot's most recent save.
+	string FileDate(const string &filename)
+	{
+		string date = "0000-00-00";
+		DataFile file(filename);
+		for(const DataNode &node : file)
+			if(node.Token(0) == "date")
+			{
+				int year = node.Value(3);
+				int month = node.Value(2);
+				int day = node.Value(1);
+				date[0] += (year / 1000) % 10;
+				date[1] += (year / 100) % 10;
+				date[2] += (year / 10) % 10;
+				date[3] += year % 10;
+				date[5] += (month / 10) % 10;
+				date[6] += month % 10;
+				date[8] += (day / 10) % 10;
+				date[9] += day % 10;
+				break;
+			}
+		return date;
+	}
+	
+	// Only show tooltips if the mouse has hovered in one place for this amount
+	// of time.
+	const int HOVER_TIME = 60;
 }
 
 
@@ -60,11 +95,13 @@ namespace {
 LoadPanel::LoadPanel(PlayerInfo &player, UI &gamePanels)
 	: player(player), gamePanels(gamePanels), selectedPilot(player.Identifier())
 {
-	// If you have a player loaded, and the player is on a planet, makes sure
+	// If you have a player loaded, and the player is on a planet, make sure
 	// the player is saved so that any snapshot you create will be of the
 	// player's current state, rather than one planet ago. Only do this if the
-	// game is paused, i.e. the "main panel" is not on top:
-	if(player.GetPlanet() && !player.IsDead() && !gamePanels.IsTop(&*gamePanels.Root()))
+	// game is paused and 'dirty', i.e. the "main panel" is not on top, and we
+	// actually were using the loaded save.
+	if(player.GetPlanet() && !player.IsDead() && !gamePanels.IsTop(&*gamePanels.Root())
+			&& gamePanels.CanSave())
 		player.Save();
 	UpdateLists();
 }
@@ -75,6 +112,7 @@ void LoadPanel::Draw()
 {
 	glClear(GL_COLOR_BUFFER_BIT);
 	GameData::Background().Draw(Point(), Point());
+	const Font &font = FontSet::Get(14);
 	
 	Information info;
 	if(loadedInfo.IsLoaded())
@@ -91,6 +129,7 @@ void LoadPanel::Draw()
 			info.SetString("planet", loadedInfo.GetPlanet());
 		info.SetString("credits", loadedInfo.Credits());
 		info.SetString("date", loadedInfo.GetDate());
+		info.SetString("playtime", loadedInfo.GetPlayTime());
 	}
 	else
 		info.SetString("pilot", "No Pilot Loaded");
@@ -108,21 +147,19 @@ void LoadPanel::Draw()
 	GameData::Interfaces().Get("load menu")->Draw(info, this);
 	GameData::Interfaces().Get("menu player info")->Draw(info, this);
 	
-	const Font &font = FontSet::Get(14);
-	
 	// The list has space for 14 entries. Alpha should be 100% for Y = -157 to
 	// 103, and fade to 0 at 10 pixels beyond that.
 	Point point(-470., -157. - sideScroll);
+	const double centerY = font.Height() / 2;
 	for(const auto &it : files)
 	{
-		Rectangle zone(point + Point(110., 7.), Point(230., 20.));
+		Rectangle zone(point + Point(110., centerY), Point(230., 20.));
 		bool isHighlighted = (it.first == selectedPilot || (hasHover && zone.Contains(hoverPoint)));
 		
 		double alpha = min(1., max(0., min(.1 * (113. - point.Y()), .1 * (point.Y() - -167.))));
 		if(it.first == selectedPilot)
 			FillShader::Fill(zone.Center(), zone.Dimensions(), Color(.1 * alpha, 0.));
-		string name = font.Truncate(it.first, 220);
-		font.Draw(name, point, Color((isHighlighted ? .7 : .5) * alpha, 0.));
+		font.Draw({it.first, {220, Truncate::BACK}}, point, Color((isHighlighted ? .7 : .5) * alpha, 0.));
 		point += Point(0., 20.);
 	}
 	
@@ -137,13 +174,12 @@ void LoadPanel::Draw()
 		for(const auto &it : files.find(selectedPilot)->second)
 		{
 			const string &file = it.first;
-			Rectangle zone(point + Point(110., 7.), Point(230., 20.));
+			Rectangle zone(point + Point(110., centerY), Point(230., 20.));
 			double alpha = min(1., max(0., min(.1 * (113. - point.Y()), .1 * (point.Y() - -167.))));
 			bool isHovering = (alpha && hasHover && zone.Contains(hoverPoint));
 			bool isHighlighted = (file == selectedFile || isHovering);
 			if(isHovering)
 			{
-				static const int HOVER_TIME = 60;
 				hoverCount = min(HOVER_TIME, hoverCount + 2);
 				if(hoverCount == HOVER_TIME)
 					hoverText = TimestampString(it.second);
@@ -152,8 +188,8 @@ void LoadPanel::Draw()
 			if(file == selectedFile)
 				FillShader::Fill(zone.Center(), zone.Dimensions(), Color(.1 * alpha, 0.));
 			size_t pos = file.find('~') + 1;
-			string name = font.Truncate(file.substr(pos, file.size() - 4 - pos), 220);
-			font.Draw(name, point, Color((isHighlighted ? .7 : .5) * alpha, 0.));
+			const string name = file.substr(pos, file.size() - 4 - pos);
+			font.Draw({name, {220, Truncate::BACK}}, point, Color((isHighlighted ? .7 : .5) * alpha, 0.));
 			point += Point(0., 20.);
 		}
 	}
@@ -161,39 +197,39 @@ void LoadPanel::Draw()
 	{
 		Point boxSize(font.Width(hoverText) + 20., 30.);
 		
-		FillShader::Fill(hoverPoint + .5 * boxSize, boxSize, Color(.3, 1.));
-		font.Draw(hoverText, hoverPoint + Point(10., 10.), Color(.5, 0.));
+		FillShader::Fill(hoverPoint + .5 * boxSize, boxSize, *GameData::Colors().Get("tooltip background"));
+		font.Draw(hoverText, hoverPoint + Point(10., 10.), *GameData::Colors().Get("medium"));
 	}
 }
 
 
 
-bool LoadPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
+bool LoadPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
 	if(key == 'n')
 	{
-		player.New();
-		
-		ConversationPanel *panel = new ConversationPanel(
-			player, *GameData::Conversations().Get("intro"));
-		GetUI()->Push(panel);
-		panel->SetCallback(this, &LoadPanel::OnCallback);
+		// If no player is loaded, the "Enter Ship" button becomes "New Pilot."
+		// Request that the player chooses a start scenario.
+		// StartConditionsPanel also handles the case where there's no scenarios.
+		GetUI()->Push(new StartConditionsPanel(player, gamePanels, GameData::StartOptions(), this));
 	}
 	else if(key == 'D' && !selectedPilot.empty())
 	{
 		GetUI()->Push(new Dialog(this, &LoadPanel::DeletePilot,
-			"Are you sure you want to delete the selected pilot, \""
-				+ selectedPilot + "\", and all their saved games?"));
+			"Are you sure you want to delete the selected pilot, \"" + selectedPilot
+				+ "\", and all their saved games?\n\n(This will permanently delete the pilot data.)"));
 	}
 	else if(key == 'a' && !player.IsDead() && player.IsLoaded())
 	{
-		string wasSelected = selectedPilot;
 		auto it = files.find(selectedPilot);
 		if(it == files.end() || it->second.empty() || it->second.front().first.size() < 4)
 			return false;
 		
+		nameToConfirm.clear();
+		string lastSave = Files::Saves() + it->second.front().first;
 		GetUI()->Push(new Dialog(this, &LoadPanel::SnapshotCallback,
-			"Enter a name for this snapshot, or leave the name empty to use the current date:"));
+			"Enter a name for this snapshot, or use the most recent save's date:",
+			FileDate(lastSave)));
 	}
 	else if(key == 'R' && !selectedFile.empty())
 	{
@@ -279,6 +315,9 @@ bool LoadPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 
 bool LoadPanel::Click(int x, int y, int clicks)
 {
+	// When the user clicks, clear the hovered state.
+	hasHover = false;
+	
 	// The first row of each panel is y = -160 to -140.
 	if(y < -160 || y >= (-160 + 14 * 20))
 		return false;
@@ -292,6 +331,7 @@ bool LoadPanel::Click(int x, int y, int clicks)
 			{
 				selectedPilot = it.first;
 				selectedFile = it.second.front().first;
+				centerScroll = 0;
 			}
 	}
 	else if(x >= -110 && x < 110)
@@ -306,7 +346,7 @@ bool LoadPanel::Click(int x, int y, int clicks)
 			{
 				selectedFile = it.first;
 				if(clicks > 1)
-					KeyDown('l', 0, Command());
+					KeyDown('l', 0, Command(), true);
 				break;
 			}
 	}
@@ -330,6 +370,11 @@ bool LoadPanel::Hover(int x, int y)
 	
 	hasHover = true;
 	hoverPoint = Point(x, y);
+	// Tooltips should not pop up unless the mouse stays in one place for the
+	// full hover time. Otherwise, every time the user scrubs the mouse over the
+	// list, tooltips will appear after one second.
+	if(hoverCount < HOVER_TIME)
+		hoverCount = 0;
 	
 	return true;
 }
@@ -363,7 +408,7 @@ void LoadPanel::UpdateLists()
 	for(const string &path : fileList)
 	{
 		string fileName = Files::Name(path);
-		// The file name is either "Pilot Name.txt" or "Pilot Name~Date.txt".
+		// The file name is either "Pilot Name.txt" or "Pilot Name~SnapshotTitle.txt".
 		size_t pos = fileName.find('~');
 		if(pos == string::npos)
 			pos = fileName.size() - 4;
@@ -398,64 +443,45 @@ void LoadPanel::UpdateLists()
 
 
 
-// New player "conversation" callback.
-void LoadPanel::OnCallback(int)
-{
-	GetUI()->Pop(this);
-	GetUI()->Pop(GetUI()->Root().get());
-	gamePanels.Reset();
-	gamePanels.Push(new MainPanel(player));
-	// Tell the main panel to re-draw itself (and pop up the planet panel).
-	gamePanels.StepAll();
-	gamePanels.Push(new ShipyardPanel(player));
-	gamePanels.StepAll();
-}
-
-
-
 // Snapshot name callback.
 void LoadPanel::SnapshotCallback(const string &name)
 {
-	string wasSelected = selectedPilot;
 	auto it = files.find(selectedPilot);
 	if(it == files.end() || it->second.empty() || it->second.front().first.size() < 4)
 		return;
 	
 	string from = Files::Saves() + it->second.front().first;
-	string extension = "~" + name + ".txt";
-	if(name.empty())
-	{
-		// Extract the date from this pilot's most recent save.
-		extension = "~0000-00-00.txt";
-		DataFile file(from);
-		for(const DataNode &node : file)
-			if(node.Token(0) == "date")
-			{
-				int year = node.Value(3);
-				int month = node.Value(2);
-				int day = node.Value(1);
-				extension[1] += (year / 1000) % 10;
-				extension[2] += (year / 100) % 10;
-				extension[3] += (year / 10) % 10;
-				extension[4] += year % 10;
-				extension[6] += (month / 10) % 10;
-				extension[7] += month % 10;
-				extension[9] += (day / 10) % 10;
-				extension[10] += day % 10;
-			}
-	}
+	string suffix = name.empty() ? FileDate(from) : name;
+	string extension = "~" + suffix + ".txt";
 	
-	// Copy the autosave to a new, named file.
+	// If a file with this name already exists, make sure the player
+	// actually wants to overwrite it.
 	string to = from.substr(0, from.size() - 4) + extension;
-	Files::Copy(from, to);
-	if(Files::Exists(to))
+	if(Files::Exists(to) && suffix != nameToConfirm)
+	{
+		nameToConfirm = suffix;
+		GetUI()->Push(new Dialog(this, &LoadPanel::SnapshotCallback, "Warning: \"" + suffix
+			+ "\" is being used for an existing snapshot.\nOverwrite it?", suffix));
+	}
+	else
+		WriteSnapshot(from, to);
+}
+
+
+
+// This name is the one to be used, even if it already exists.
+void LoadPanel::WriteSnapshot(const string &sourceFile, const string &snapshotName)
+{
+	// Copy the autosave to a new, named file.
+	Files::Copy(sourceFile, snapshotName);
+	if(Files::Exists(snapshotName))
 	{
 		UpdateLists();
-		selectedFile = Files::Name(to);
+		selectedFile = Files::Name(snapshotName);
 		loadedInfo.Load(Files::Saves() + selectedFile);
 	}
 	else
-		GetUI()->Push(new Dialog("Error: unable to create the file \"" + to + "\"."));
+		GetUI()->Push(new Dialog("Error: unable to create the file \"" + snapshotName + "\"."));
 }
 
 
@@ -466,6 +492,7 @@ void LoadPanel::LoadCallback()
 	// First, make sure the previous MainPanel has been deleted, so
 	// its background thread is no longer running.
 	gamePanels.Reset();
+	gamePanels.CanSave(true);
 	
 	player.Load(loadedInfo.Path());
 	
