@@ -15,9 +15,9 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Command.h"
 #include "ConversationPanel.h"
 #include "Files.h"
-#include "Font.h"
-#include "FontSet.h"
-#include "Format.h"
+#include "text/Font.h"
+#include "text/FontSet.h"
+#include "text/Format.h"
 #include "GameData.h"
 #include "Interface.h"
 #include "Information.h"
@@ -28,19 +28,25 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Point.h"
 #include "PointerShader.h"
 #include "PreferencesPanel.h"
+#include "Ship.h"
 #include "ShipyardPanel.h"
 #include "Sprite.h"
 #include "SpriteShader.h"
 #include "StarField.h"
+#include "StartConditionsPanel.h"
 #include "System.h"
 #include "UI.h"
 
+#include "gl_header.h"
+
+#include <algorithm>
+#include <stdexcept>
+
 using namespace std;
 
 namespace {
-	static bool isReady = false;
-	static float alpha = 1.;
-	static const int scrollSpeed = 2;
+	float alpha = 1.f;
+	const int scrollSpeed = 2;
 }
 
 
@@ -57,24 +63,20 @@ MenuPanel::MenuPanel(PlayerInfo &player, UI &gamePanels)
 
 void MenuPanel::Step()
 {
-	if(GetUI()->IsTop(this) && alpha < 1.)
+	if(GetUI()->IsTop(this) && alpha < 1.f)
 	{
 		++scroll;
 		if(scroll >= (20 * credits.size() + 300) * scrollSpeed)
 			scroll = 0;
 	}
 	progress = static_cast<int>(GameData::Progress() * 60.);
-	if(progress == 60 && !isReady)
+	if(GameData::IsLoaded() && gamePanels.IsEmpty())
 	{
-		if(gamePanels.IsEmpty())
-		{
-			gamePanels.Push(new MainPanel(player));
-			// It takes one step to figure out the planet panel should be created, and
-			// another step to actually place it. So, take two steps to avoid a flicker.
-			gamePanels.StepAll();
-			gamePanels.StepAll();
-		}
-		isReady = true;
+		gamePanels.Push(new MainPanel(player));
+		// It takes one step to figure out the planet panel should be created, and
+		// another step to actually place it. So, take two steps to avoid a flicker.
+		gamePanels.StepAll();
+		gamePanels.StepAll();
 	}
 }
 
@@ -84,6 +86,7 @@ void MenuPanel::Draw()
 {
 	glClear(GL_COLOR_BUFFER_BIT);
 	GameData::Background().Draw(Point(), Point());
+	const Font &font = FontSet::Get(14);
 	
 	Information info;
 	if(player.IsLoaded() && !player.IsDead())
@@ -100,8 +103,9 @@ void MenuPanel::Draw()
 			info.SetString("system", player.GetSystem()->Name());
 		if(player.GetPlanet())
 			info.SetString("planet", player.GetPlanet()->Name());
-		info.SetString("credits", Format::Number(player.Accounts().Credits()));
+		info.SetString("credits", Format::Credits(player.Accounts().Credits()));
 		info.SetString("date", player.GetDate().ToString());
+		info.SetString("playtime", Format::PlayTime(player.GetPlayTime()));
 	}
 	else if(player.IsLoaded())
 	{
@@ -127,13 +131,12 @@ void MenuPanel::Draw()
 		Angle a(0.);
 		for(int i = 0; i < progress; ++i)
 		{
-			Color color(.5 * alpha, 0.f);
-			PointerShader::Draw(Point(), a.Unit(), 8., 20., 140. * alpha, color);
+			Color color(.5f * alpha, 0.f);
+			PointerShader::Draw(Point(), a.Unit(), 8.f, 20.f, 140.f * alpha, color);
 			a += da;
 		}
 	}
 	
-	const Font &font = FontSet::Get(14);
 	int y = 120 - scroll / scrollSpeed;
 	for(const string &line : credits)
 	{
@@ -144,7 +147,7 @@ void MenuPanel::Draw()
 			fade = max(0.f, (115 - y) / 20.f);
 		if(fade)
 		{
-			Color color(((line.empty() || line[0] == ' ') ? .2 : .4) * fade, 0.);
+			Color color(((line.empty() || line[0] == ' ') ? .2f : .4f) * fade, 0.f);
 			font.Draw(line, Point(-470., y), color);
 		}
 		y += 20;
@@ -153,27 +156,16 @@ void MenuPanel::Draw()
 
 
 
-// New player "conversation" callback.
-void MenuPanel::OnCallback(int)
+bool MenuPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
-	GetUI()->Pop(this);
-	gamePanels.Reset();
-	gamePanels.Push(new MainPanel(player));
-	// Tell the main panel to re-draw itself (and pop up the planet panel).
-	gamePanels.StepAll();
-	gamePanels.Push(new ShipyardPanel(player));
-	gamePanels.StepAll();
-}
-
-
-
-bool MenuPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
-{
-	if(!isReady)
+	if(!GameData::IsLoaded())
 		return false;
 	
 	if(player.IsLoaded() && (key == 'e' || command.Has(Command::MENU)))
+	{
+		gamePanels.CanSave(true);
 		GetUI()->Pop(this);
+	}
 	else if(key == 'p')
 		GetUI()->Push(new PreferencesPanel());
 	else if(key == 'l')
@@ -181,12 +173,9 @@ bool MenuPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 	else if(key == 'n' && (!player.IsLoaded() || player.IsDead()))
 	{
 		// If no player is loaded, the "Enter Ship" button becomes "New Pilot."
-		player.New();
-		
-		ConversationPanel *panel = new ConversationPanel(
-			player, *GameData::Conversations().Get("intro"));
-		GetUI()->Push(panel);
-		panel->SetCallback(this, &MenuPanel::OnCallback);
+		// Request that the player chooses a start scenario.
+		// StartConditionsPanel also handles the case where there's no scenarios.
+		GetUI()->Push(new StartConditionsPanel(player, gamePanels, GameData::StartOptions(), nullptr));
 	}
 	else if(key == 'q')
 		GetUI()->Quit();
