@@ -13,7 +13,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "OutfitInfoDisplay.h"
 
 #include "Depreciation.h"
-#include "Format.h"
+#include "text/Format.h"
 #include "Outfit.h"
 #include "PlayerInfo.h"
 
@@ -26,50 +26,97 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 using namespace std;
 
 namespace {
-	const map<string, double> SCALE = {
-		{"active cooling", 60.},
-		{"afterburner energy", 60.},
-		{"afterburner fuel", 60.},
-		{"afterburner heat", 60.},
-		{"cloak", 60.},
-		{"cloaking energy", 60.},
-		{"cloaking fuel", 60.},
-		{"cloaking heat", 60.},
-		{"cooling", 60.},
-		{"cooling energy", 60.},
-		{"energy consumption", 60.},
-		{"energy generation", 60.},
-		{"fuel consumption", 60.},
-		{"fuel energy", 60.},
-		{"fuel generation", 60.},
-		{"fuel heat", 60.},
-		{"heat generation", 60.},
-		{"heat dissipation", 60.},
-		{"hull repair rate", 60.},
-		{"hull energy", 60.},
-		{"hull fuel", 60.},
-		{"hull heat", 60.},
-		{"jump speed", 60.},
-		{"reverse thrusting energy", 60.},
-		{"reverse thrusting heat", 60.},
-		{"shield generation", 60.},
-		{"shield energy", 60.},
-		{"shield fuel", 60.},
-		{"shield heat", 60.},
-		{"solar collection", 60.},
-		{"thrusting energy", 60.},
-		{"thrusting heat", 60.},
-		{"turn", 60.},
-		{"turning energy", 60.},
-		{"turning heat", 60.},
+	const vector<pair<double, string>> SCALE_LABELS = {
+		make_pair(60., ""),
+		make_pair(60. * 60., ""),
+		make_pair(60. * 100., ""),
+		make_pair(100., "%"),
+		make_pair(100., ""),
+		make_pair(1. / 60., "")
+	};
+	
+	const map<string, int> SCALE = {
+		{"active cooling", 0},
+		{"afterburner energy", 0},
+		{"afterburner fuel", 0},
+		{"afterburner heat", 0},
+		{"cloak", 0},
+		{"cloaking energy", 0},
+		{"cloaking fuel", 0},
+		{"cloaking heat", 0},
+		{"cooling", 0},
+		{"cooling energy", 0},
+		{"disruption resistance energy", 0},
+		{"disruption resistance fuel", 0},
+		{"disruption resistance heat", 0},
+		{"energy consumption", 0},
+		{"energy generation", 0},
+		{"fuel consumption", 0},
+		{"fuel energy", 0},
+		{"fuel generation", 0},
+		{"fuel heat", 0},
+		{"heat generation", 0},
+		{"heat dissipation", 0},
+		{"hull repair rate", 0},
+		{"hull energy", 0},
+		{"hull fuel", 0},
+		{"hull heat", 0},
+		{"ion resistance energy", 0},
+		{"ion resistance fuel", 0},
+		{"ion resistance heat", 0},
+		{"jump speed", 0},
+		{"reverse thrusting energy", 0},
+		{"reverse thrusting heat", 0},
+		{"scram drive", 0},
+		{"shield generation", 0},
+		{"shield energy", 0},
+		{"shield fuel", 0},
+		{"shield heat", 0},
+		{"slowing resistance energy", 0},
+		{"slowing resistance fuel", 0},
+		{"slowing resistance heat", 0},
+		{"solar collection", 0},
+		{"solar heat", 0},
+		{"thrusting energy", 0},
+		{"thrusting heat", 0},
+		{"turn", 0},
+		{"turning energy", 0},
+		{"turning heat", 0},
 		
-		{"thrust", 60. * 60.},
-		{"reverse thrust", 60. * 60.},
-		{"afterburner thrust", 60. * 60.},
+		{"thrust", 1},
+		{"reverse thrust", 1},
+		{"afterburner thrust", 1},
 		
-		{"ion resistance", 60. * 100.},
-		{"disruption resistance", 60. * 100.},
-		{"slowing resistance", 60. * 100.}
+		{"ion resistance", 2},
+		{"disruption resistance", 2},
+		{"slowing resistance", 2},
+		
+		{"hull repair multiplier", 3},
+		{"hull energy multiplier", 3},
+		{"hull fuel multiplier", 3},
+		{"hull heat multiplier", 3},
+		{"piercing resistance", 3},
+		{"shield generation multiplier", 3},
+		{"shield energy multiplier", 3},
+		{"shield fuel multiplier", 3},
+		{"shield heat multiplier", 3},
+		{"threshold percentage", 3},
+		
+		{"disruption protection", 4},
+		{"energy protection", 4},
+		{"force protection", 4},
+		{"fuel protection", 4},
+		{"heat protection", 4},
+		{"hull protection", 4},
+		{"ion protection", 4},
+		{"piercing protection", 4},
+		{"shield protection", 4},
+		{"slowing protection", 4},
+		
+		{"repair delay", 5},
+		{"disabled repair delay", 5},
+		{"shield delay", 5},
+		{"depleted shield delay", 5}
 	};
 	
 	const map<string, string> BOOLEAN_ATTRIBUTES = {
@@ -205,7 +252,8 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 			continue;
 		
 		auto sit = SCALE.find(it.first);
-		double scale = (sit == SCALE.end() ? 1. : sit->second);
+		double scale = (sit == SCALE.end() ? 1. : SCALE_LABELS[sit->second].first);
+		string units = (sit == SCALE.end() ? "" : SCALE_LABELS[sit->second].second);
 		
 		auto bit = BOOLEAN_ATTRIBUTES.find(it.first);
 		if(bit != BOOLEAN_ATTRIBUTES.end()) 
@@ -217,7 +265,7 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		else
 		{
 			attributeLabels.emplace_back(static_cast<string>(it.first) + ":");
-			attributeValues.emplace_back(Format::Number(it.second * scale));
+			attributeValues.emplace_back(Format::Number(it.second * scale) + units);
 			attributesHeight += 20;
 		}
 		hasNormalAttributes = true;
@@ -239,23 +287,45 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		attributeLabels.emplace_back("ammo:");
 		attributeValues.emplace_back(outfit.Ammo()->Name());
 		attributesHeight += 20;
+		if(outfit.AmmoUsage() != 1)
+		{
+			attributeLabels.emplace_back("ammo usage:");
+			attributeValues.emplace_back(Format::Number(outfit.AmmoUsage()));
+			attributesHeight += 20;
+		}
 	}
 	
 	attributeLabels.emplace_back("range:");
 	attributeValues.emplace_back(Format::Number(outfit.Range()));
 	attributesHeight += 20;
 	
-	static const vector<string> VALUE_NAMES = {
-		"shield damage",
-		"hull damage",
-		"fuel damage",
-		"heat damage",
-		"ion damage",
-		"slowing damage",
-		"disruption damage",
-		"firing energy",
-		"firing heat",
-		"firing fuel"
+	static const vector<pair<string, string>> VALUE_NAMES = {
+		{"shield damage", ""},
+		{"hull damage", ""},
+		{"fuel damage", ""},
+		{"heat damage", ""},
+		{"energy damage", ""},
+		{"ion damage", ""},
+		{"slowing damage", ""},
+		{"disruption damage", ""},
+		{"% shield damage", "%"},
+		{"% hull damage", "%"},
+		{"% fuel damage", "%"},
+		{"% heat damage", "%"},
+		{"% energy damage", "%"},
+		{"firing energy", ""},
+		{"firing heat", ""},
+		{"firing fuel", ""},
+		{"firing hull", ""},
+		{"firing shields", ""},
+		{"firing ion", ""},
+		{"firing slowing", ""},
+		{"firing disruption", ""},
+		{"% firing energy", "%"},
+		{"% firing heat", "%"},
+		{"% firing fuel", "%"},
+		{"% firing hull", "%"},
+		{"% firing shields", "%"}
 	};
 	
 	vector<double> values = {
@@ -263,12 +333,28 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		outfit.HullDamage(),
 		outfit.FuelDamage(),
 		outfit.HeatDamage(),
+		outfit.EnergyDamage(),
 		outfit.IonDamage() * 100.,
 		outfit.SlowingDamage() * 100.,
 		outfit.DisruptionDamage() * 100.,
+		outfit.RelativeShieldDamage() * 100.,
+		outfit.RelativeHullDamage() * 100.,
+		outfit.RelativeFuelDamage() * 100.,
+		outfit.RelativeHeatDamage() * 100.,
+		outfit.RelativeEnergyDamage() * 100.,
 		outfit.FiringEnergy(),
 		outfit.FiringHeat(),
-		outfit.FiringFuel()
+		outfit.FiringFuel(),
+		outfit.FiringHull(),
+		outfit.FiringShields(),
+		outfit.FiringIon() * 100.,
+		outfit.FiringSlowing() * 100.,
+		outfit.FiringDisruption() * 100.,
+		outfit.RelativeFiringEnergy() * 100.,
+		outfit.RelativeFiringHeat() * 100.,
+		outfit.RelativeFiringFuel() * 100.,
+		outfit.RelativeFiringHull() * 100.,
+		outfit.RelativeFiringShields() * 100.
 	};
 	
 	// Add any per-second values to the table.
@@ -279,8 +365,8 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		for(unsigned i = 0; i < values.size(); ++i)
 			if(values[i])
 			{
-				attributeLabels.emplace_back(VALUE_NAMES[i] + PER_SECOND);
-				attributeValues.emplace_back(Format::Number(60. * values[i] / reload));
+				attributeLabels.emplace_back(VALUE_NAMES[i].first + PER_SECOND);
+				attributeValues.emplace_back(Format::Number(60. * values[i] / reload) + VALUE_NAMES[i].second);
 				attributesHeight += 20;
 			}
 	}
@@ -350,8 +436,8 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		for(unsigned i = 0; i < VALUE_NAMES.size(); ++i)
 			if(values[i])
 			{
-				attributeLabels.emplace_back(VALUE_NAMES[i] + PER_SHOT);
-				attributeValues.emplace_back(Format::Number(values[i]));
+				attributeLabels.emplace_back(VALUE_NAMES[i].first + PER_SHOT);
+				attributeValues.emplace_back(Format::Number(values[i]) + VALUE_NAMES[i].second);
 				attributesHeight += 20;
 			}
 	}
