@@ -17,10 +17,12 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include <functional>
 #include <list>
+#include <string>
 
 #include <SDL2/SDL.h>
 
 class Command;
+class Point;
 class UI;
 
 
@@ -33,7 +35,7 @@ class UI;
 class Panel {
 public:
 	// Make the destructor virtual just in case any derived class needs it.
-	virtual ~Panel();
+	virtual ~Panel() = default;
 	
 	// Move the state of this panel forward one game step.
 	virtual void Step();
@@ -59,10 +61,13 @@ public:
 	// so, apply that zone's action and return true.
 	bool ZoneClick(const Point &point);
 	
+	// Is fast-forward allowed to be on when this panel is on top of the GUI stack?
+	virtual bool AllowFastForward() const;
+	
 	
 protected:
 	// Only override the ones you need; the default action is to return false.
-	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command);
+	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress);
 	virtual bool Click(int x, int y, int clicks);
 	virtual bool RClick(int x, int y);
 	virtual bool Hover(int x, int y);
