@@ -20,7 +20,7 @@ using namespace std;
 
 
 
-Point::Point()
+Point::Point() noexcept
 #ifdef __SSE3__
 	: v(_mm_setzero_pd())
 #else
@@ -31,7 +31,7 @@ Point::Point()
 
 
 
-Point::Point(double x, double y)
+Point::Point(double x, double y) noexcept
 #ifdef __SSE3__
 	: v(_mm_set_pd(y, x))
 #else
@@ -42,39 +42,15 @@ Point::Point(double x, double y)
 
 
 
-Point::Point(const Point &point)
-#ifdef __SSE3__
-	: v(point.v)
-#else
-	: x(point.x), y(point.y)
-#endif
-{
-}
-
-
-
-Point &Point::operator=(const Point &point)
-{
-#ifdef __SSE3__
-	v = point.v;
-#else
-	x = point.x;
-	y = point.y;
-#endif
-	return *this;
-}
-
-
-
 // Check if the point is anything but (0, 0).
-Point::operator bool() const
+Point::operator bool() const noexcept
 {
 	return !!*this;
 }
 
 
 
-bool Point::operator!() const
+bool Point::operator!() const noexcept
 {
 	return (!x & !y);
 }
@@ -322,7 +298,7 @@ Point abs(const Point &p)
 #ifdef __SSE3__
 	// Absolute value for doubles just involves clearing the sign bit.
 	static const __m128d sign_mask = _mm_set1_pd(-0.);
-    return Point(_mm_andnot_pd(sign_mask, p.v));
+	return Point(_mm_andnot_pd(sign_mask, p.v));
 #else
 	return Point(abs(p.x), abs(p.y));
 #endif
