@@ -21,13 +21,16 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 // whether it is looping (ends in '~') or not.
 class Sound {
 public:
-	bool Load(const std::string &path);
+	bool Load(const std::string &path, const std::string &name);
+	
+	const std::string &Name() const;
 	
 	unsigned Buffer() const;
 	bool IsLooping() const;
 	
 	
 private:
+	std::string name;
 	unsigned buffer = 0;
 	bool isLooped = false;
 };
